# -*- coding: utf-8 -*-

# This script creates a csv file that contains the Sentinel-1 tile 
# names and their download links of the BigEarthNet Sentinel-1 v1.0 patches.
# 
# The script is capable of searching corresponding tiles of the BigEarthNet 
# patches in Copernicus Open Access Hub via sentinelsat API. 
# 
# python extract_tile_names_and_links.py --help can be used to learn 
# how to use this script.
#
# Date: 14 May 2019
# Version: 1.0.2
# Usage: extract_tile_names_and_links.py -u USERNAME -p PASSWORD
#                                        -r ROOT_FOLDER -o OUT_PATH

from __future__ import print_function
from datetime import datetime, timedelta
import json
import os
import argparse
import csv

parser = argparse.ArgumentParser(
    description='This script extracts the tile names and ' + 
    'their download links for the BigEarthNet Sentinel-1 v1.0 image patches')
parser.add_argument('-u', '--username', dest='username', help='username ' + 
                    'of a Copernicus Open Access Hub account')
parser.add_argument('-p', '--password', dest='password', help='password ' + 
                    'of a Copernicus Open Access Hub account')
parser.add_argument('-r', '--root_folder', dest='root_folder',
                    help='root folder path contains multiple patch folders')
parser.add_argument('-o', '--out_path', dest='out_path',
                    help='file path of the output csv file whose each ' + 
                    'row contains patch name, tile name and download link')
args = parser.parse_args()

if (not args.username) or (not args.password) or (not args.out_path):
        print('ERROR: need to specify -u, -p, -u and -o arguments')
        print('INFO: For help, python extract_tile_names_and_links.py --help')
        exit()
elif args.root_folder:
    if not os.path.exists(args.root_folder):
        print('ERROR: folder', args.root_folder, 'does not exist')
        print('INFO: need to download archive first')
        exit()
    root_folder = os.path.realpath(args.root_folder)

# Checks the existence of required python packages
try:
    from sentinelsat import SentinelAPI
except ImportError:
    print('ERROR: please install sentinelsat')
    print('INFO: for how to install sentinelsat, see ' + 
    'https://github.com/sentinelsat/sentinelsat')
    exit()
try:
    from os import scandir
except ImportError:
    try:
        from scandir import scandir
    except ImportError:
        print('ERROR: Either use Python 3 or ' + 
        'install scandir package for Python 2')
        exit()

# Creates SentinelAPI object with username and password
api = SentinelAPI(args.username, args.password)

dict_iter_items = (lambda d: d.iteritems()) if hasattr(
    dict, 'iteritems') else (lambda d: iter(d.items()))
search_res = {}

# Creates resulting csv file
with open(os.path.realpath(args.out_path), 'w') as fp:
    csv_writer = csv.writer(fp)
    # Iterates over patch folders
    for folder in scandir(root_folder):
        send_query = False
        patch_folder_path = os.path.realpath(folder.path)
        patch_name = os.path.basename(patch_folder_path)
        labels_metadata_path = os.path.join(patch_folder_path, 
                            patch_name + '_labels_metadata.json')
        
        # Reads labels_metadata json file 
        with open(labels_metadata_path, 'r') as f:
            labels_metadata = json.load(f)

        scene_identifier = labels_metadata["scene_source"]
        scene_filename = scene_identifier + ".SAFE"

        if not scene_filename in search_res:
            query_kwargs = {
                'filename': scene_filename,
                'platformname': 'Sentinel-1'
            }
            results = api.query(**query_kwargs)
            link = results.popitem()[1]["link"]
            search_res[scene_filename] = link
        else:
            link = search_res[scene_filename]

        csv_writer.writerow([patch_name, scene_filename, link])
